/*This is a stand-alone procedure that computes the eigenvalues of the MT matrix under different sample sizes.
  It produces the results in the first paragraph of Section 7.1.
  It is not required for implementing the test. */

new;
//cls;


lib user gensys.src;
#include nkmp_mod.src
#include fstats.src
#include nkmp_spec.g


dlibrary -a PszTgSen; 
outwidth 256;
format /mat /on /mb1 /ros 16,8;
_svdtol=1e-10;



msel=zeros(2,1); @selects the model@
/************************************************
*  MSEL[2]:
*   1 benchmark, output growth rule
*   3 benchmark, output gap rule
************************************************/

msel[1]=1; 
msel[2]=3; 


nn={80,160,240,320}; @the sample size@

n=nn[1];  @change to different values for different sample sizes@

steady=0; @this excludes the steady state parameters@


tau        = 2;
kap        = 0.15;

psi1       = 1.5;
psi2       = 1.00;

rhoR       = 0.60;
rhog       = 0.95;
rhoz       = 0.65;

rr_steady  = 0.40; @computed from the discount parameter beta@
pi_steady  = 4.00;
gam_steady = 0.50;

sigR       = 0.2;
sigg       = 0.8;
sigz       = 0.45;



spara=zeros(13,1);
spara[1]=tau;
spara[2]=kap;
spara[3]=psi1;
spara[4]=psi2;
spara[5]=rhoR;
spara[6]=rhog;
spara[7]=rhoz;
spara[8]=rr_steady;
spara[9]=pi_steady;
spara[10]=gam_steady;
spara[11]=sigR;
spara[12]=sigg;
spara[13]=sigz;

startf=n/32; @business cycle frequencies@
endf=n/6;

ssize=1e-6; @step size@

np=13;  @the number of parameters@

{H,b_H,Dfmat,sp_mat,sp_mat0,retcode}=HDmat2(spara,n,msel,startf,endf,steady,ssize);

H=real(H);

@exclude the steady state parameters@
H=(H[1:8,1:8]~H[1:8,11:13])|(H[11:13,1:8]~H[11:13,11:13]);
MT=H/(4*pi);

{va,ve}=eigrs2(MT);

eps=machEpsilon;
tol=eps*rows(H)*maxc(va);
print "-----the eigenvalues are:-----" va;
print "-----the default tolerance level is-----" tol;
print "-----The corresponding eigenvectors are-----" ve;


proc(6)=HDmat2(para,n,msel,startf,endf,steady,ssize);

/*
-----------------------------------------------------------------------
Input.
    para: parameter vector
    n: the sample size
    msel: model specification
    startf: the lower limit of the frequency band, can be non-integer. 
    endif: the upper limit of the frequency band
Output.
    H: the precision matrix (full spectrum, excluding the zero frequency if only the second order property is considered)
    b_H, the precision matrix (business cycle frequencies)
    Dfmat: the matrix of first derivatives
    Sp_mat: spectral density matrix, starting at frequency 0
    sp_mat0: spectral density at frequency zero, it is also the first block of sp_mat
    retcode[1]=0: model solution exists and is unique.
--------------------------------------------------------------------------
*/



local A,T1,TC,T0,Q,omi,ii,dims,dimy,sp_mat,i,exe,mat1,mat2,Hmat1,Hmat2,va,ve,H2;
local nsp_mat, dif_para,npara,nT1,nTC,nT0,nQ,nmat1,nmat2,j,Hmat,Dfmat,k,aj,ak,freq,H,isp_mat,insp_mat,iDfmat,iaj,iak;
local ij, tt,qq,hh,dd,zz,vv,rc,rr,retcode,b_H,b_Hmat2,ind,sp_mat0;

retcode=0;

{TT,QQ,RR,HH,DD,ZZ,VV,RC}=sysmat(para,msel);

if (RC[1] == 1) and (RC[2]==1);
   /* determinacy */
   retcode[1] = 0;
else;
   /* no equilibrium exists, numerical problems */
   retcode[1] = 1;
endif;

if retcode[1]==0;

dimy=3;      
//Careful! Model dependent
omi=2*pi*seqa(0,1,n)/n;     @frequency: 0 to 2pi@
ii=sqrt(-1);

sp_mat=zeros(dimy*(n),dimy);  @to store the spectral density@
isp_mat=sp_mat;               @to store the inverse@

exe=exp(-ii*0);
mat1=inv(eye(rows(TT))-TT*exe)*RR;
mat2=mat1';                   @note that ' gives the conjugate transpose@
sp_mat0=zz*mat1*QQ*mat2*zz'/(2*pi); @spectral density at frequency zero@

ij=1;
do while ij<=n;
exe=exp(-ii*omi[ij]);
mat1=inv(eye(rows(TT))-TT*exe)*RR;
mat2=mat1';                 

sp_mat[((ij-1)*dimy+1):(ij*dimy),.]=zz*mat1*QQ*mat2*zz'/(2*pi);
                            @spectral density at various frequencies@
isp_mat[((ij-1)*dimy+1):(ij*dimy),.]=pinv(sp_mat[((ij-1)*dimy+1):(ij*dimy),.]); 
                             @generalized inverse, to allow for possible singularity@
ij=ij+1;
endo;




@compute the derivative of the spectral density@
nsp_mat=zeros(dimy*(n),dimy*rows(para));

dif_para=ones(rows(para),1)*ssize; 

insp_mat=nsp_mat;
j=1;
do while j<=rows(para);

@change one parameter at a time@
npara=para;
npara[j]=para[j]-dif_para[j];  
@recompute the spectral density matrix)@
{TT,QQ,RR,HH,DD,ZZ,VV,RC}=sysmat(npara,msel);

i=1;
do while i<=n;
exe=exp(-ii*omi[i]);
mat1=inv(eye(rows(TT))-TT*exe)*RR;
mat2=mat1';                      
nsp_mat[((i-1)*dimy+1):(i*dimy),((j-1)*dimy+1):(j*dimy)]=zz*mat1*QQ*mat2*zz'/(2*pi);
insp_mat[((i-1)*dimy+1):(i*dimy),((j-1)*dimy+1):(j*dimy)]=pinv(nsp_mat[((i-1)*dimy+1):(i*dimy),((j-1)*dimy+1):(j*dimy)]);   @generalized inverse to allow for singularity@
i=i+1;
endo;
j=j+1;
endo;


@now construct the H matrix, note that the jk-th component is simply the derivative with respect to j and k@
@a simple two-point method is used@
Hmat1=zeros(rows(para),rows(para));
Hmat2=Hmat1;
b_Hmat2=Hmat2;
Dfmat=nsp_mat*0;
iDfmat=Dfmat;
j=1;
do while j<=rows(Hmat2);
k=1;
do while k<=rows(Hmat2);
aj=(sp_mat-nsp_mat[.,((j-1)*dimy+1):(j*dimy)])/dif_para[j];
Dfmat[.,((j-1)*dimy+1):(j*dimy)]=aj;
ak=(sp_mat-nsp_mat[.,((k-1)*dimy+1):(k*dimy)])/dif_para[k];

if steady==1;
freq=1;  
elseif steady==0;
freq=2;
endif;

do while freq<=(rows(sp_mat)/dimy);
Hmat2[j,k]=Hmat2[j,k]+sumc(diag(isp_mat[((freq-1)*dimy+1):(freq*dimy),.]*aj[((freq-1)*dimy+1):(freq*dimy),.]*isp_mat[((freq-1)*dimy+1):(freq*dimy),.]*ak[((freq-1)*dimy+1):(freq*dimy),.]));

@ind picks out the desired frequencies@
@note that freq=1 corresponds to the zero frequency@
ind=((freq>=startf+1) and (freq<=endf+1) ) or ((freq>=n-endf+1) and (freq<=n-startf+1) );

b_Hmat2[j,k]=b_Hmat2[j,k]+ind*sumc(diag(isp_mat[((freq-1)*dimy+1):(freq*dimy),.]*aj[((freq-1)*dimy+1):(freq*dimy),.]*isp_mat[((freq-1)*dimy+1):(freq*dimy),.]*ak[((freq-1)*dimy+1):(freq*dimy),.]));

freq=freq+1;
endo;
k=k+1;
endo;
j=j+1;

endo;

Hmat2=2*pi*Hmat2/n;     @full spectrum case@
b_Hmat2=2*pi*b_Hmat2/n; @a subset of frequencies case@


H=real(Hmat2);
b_H=real(b_Hmat2);

else;
H=0;
Dfmat=0;
sp_mat=0;
b_H=0;
sp_mat0=0;

endif;

retp(H,b_H,Dfmat,sp_mat,sp_mat0,retcode[1]);
endp;