/* This is a stand-alone procedure that computes the nonidentification curve using Qu and Tkachenko (2012)*/
/* Taking equally spaced values from the output will replicate Table 1 in the paper*/


lib user gensys.src;
#include nkmp_mod.src
#include nkmp_spec.g
#include fstats.src
dlibrary -a PszTgSen; 
outwidth 256;
format /mat /on /mb1 /ros 16,8;
_svdtol=1e-10;
cls;


/*-----------Set Direction-------------*/
Direction=2; @1: first panel; 2: second panel@


@Specify the monetary policy rule using msel@
@output gap rule@
msel = zeros(2,1);
msel[1]=1;
msel[2]=3;
tau        = 2;
kap        = 0.15;

psi1       = 1.5;
psi2       = 1.00;

rhoR       = 0.60;
rhog       = 0.95;
rhoz       = 0.65;

rr_steady  = 0.40;
pi_steady  = 4.00;
gam_steady = 0.50;

sigR       = 0.2;
sigg       = 0.8;
sigz       = 0.45;



spara=zeros(13,1);
spara[1]=tau;
spara[2]=kap;
spara[3]=psi1;
spara[4]=psi2;
spara[5]=rhoR;
spara[6]=rhog;
spara[7]=rhoz;
spara[8]=rr_steady;
spara[9]=pi_steady;
spara[10]=gam_steady;
spara[11]=sigR;
spara[12]=sigg;
spara[13]=sigz;


np=13;  @the number of parameters@
n=80  @the result is insensitive to this value@;
steady=0;

startf=n/32;
endf=n/6;

rep=1;

pstor=zeros(1000000,4);
do while rep<=1000000;


{H,b_H,Dfmat,fmat,sp_mat0,retcode}=HDmat(spara,n,msel,startf,endf,steady);

if retcode[1] ==1;
goto eofday;
endif;

H=real(H);

/*This corresponds to changing the Taylor rule parameters*/
h1=H[3,.]|H[4,.]|H[5,.]|H[11,.];
h2=h1[.,3]~h1[.,4]~h1[.,5]~h1[.,11];
{a1,b}=eigrs2(h2);

if b[1,1]>=0;
b[.,1]=b[.,1]*(-1);
endif;

if Direction==1;
b[.,1]=-b[.,1];
endif;


spara[3]=spara[3]+b[1,1]*1e-4;
spara[4]=spara[4]+b[2,1]*1e-4;
spara[5]=spara[5]+b[3,1]*1e-4;
spara[11]=spara[11]+b[4,1]*1e-4;

if spara[4]<0 or spara[3]<0;
goto eofday;
endif;

print   rep~spara[3]~spara[4]~spara[5]~spara[11]~a1[1]; @a1[1]:the smallest eigenvalue@

pstor[rep,.]=spara[3]~spara[4]~spara[5]~spara[11];


rep=rep+1;
endo;


eofday:



