/*This procedure computes the null rejection frequencies. It replicates Table 2.
 Set Rdraw=0 to replicate the first panel; set Rdraw=1 for the second panel.
 In each case, set steady=0 to get "BC frequencies" and "Full spectrum", set steady=1 to get "Mean and spectrum". 
 You should not have to modify the rest. The simulation may take a few hours to finish. */
 

new;

/*-----------------------*/
        Rdraw=0;
        steady=0;
/*-----------------------*/

ltheta ="pa";
open fhltheta = ^ltheta for read;
thetasim = readr( fhltheta, 11000 );

lib user gensys.src;
#include nkmp_mod.src
#include nkmp_spec.g
#include nkmp_sim.src
#include fstats.src

library pgraph, user;

dlibrary -a PszTgSen; 
outwidth 256;
format /mat /on /mb1 /ros 16,8;
format /ld 6,4;
_svdtol=1e-10;


rndseed 12345; @seed for random numbers@
/************************************************
*  MSEL[2]:
*   1 benchmark, output growth rule
*   3 benchmark, output gap rule
************************************************/
msel=zeros(2,1);
msel[1]=1; 
msel[2]=3; 




nn={80,160,240,320}; @sample size@

jk=1;
do while jk<=4;


rejfo=0;  @store the rejection frequencies@
rejfbo=0;
rejfo90=0;
rejfbo90=0;


pdraw=1;
simu=pdraw;
do while pdraw<=5000;
pdraw=simu;

if Rdraw==1;
spara=thetasim[pdraw,.];
spara=spara';

elseif Rdraw==0;
tau        = 2;
kap        = 0.15;

psi1       = 1.5;
psi2       = 1.00;

rhoR       = 0.60;
rhog       = 0.95;
rhoz       = 0.65;

rr_steady  = 0.40;
pi_steady  = 4.00;
gam_steady = 0.50;

sigR       = 0.2;
sigg       = 0.8;
sigz       = 0.45;
spara=zeros(13,1);
spara[1]=tau;
spara[2]=kap;
spara[3]=psi1;
spara[4]=psi2;
spara[5]=rhoR;
spara[6]=rhog;
spara[7]=rhoz;
spara[8]=rr_steady;
spara[9]=pi_steady;
spara[10]=gam_steady;
spara[11]=sigR;
spara[12]=sigg;
spara[13]=sigz;
else;
print "set Rdraw to 1 or zero";
endif;


np=13;  @the number of parameters@


dpara=spara;

@generating simulated series using AS's code@
yt = dsgesim(spara,msel,Tsimburn,Tsimtotal); 

yt=yt[Tsimburn+1:Tsimtotal,.];
yt=yt[1:nn[jk],.];
n=rows(yt);     @sample size@

startf=round(n/32); @the frequency band@
endf=round(n/6);

dimy=cols(yt);  @number of variables@


{H,b_H,Dfmat,fmat,sp_mat0,retcode}=HDmat(dpara,n,msel,startf,endf,steady);
      if retcode==0;

      mu=zeros(3,1);
      mu[1]=dpara[10];
      mu[2]=dpara[9];
      mu[3]=dpara[8]+dpara[9]+4*dpara[10];
      yy=yt; 
      yy[.,1]=yt[.,1]-mu[1]; @demean@
      yy[.,2]=yt[.,2]-mu[2];
      yy[.,3]=yt[.,3]-mu[3];

      @compute the periodogram, starting at frequency zero@
      perd=zeros(dimy*n,dimy);
      dy=zeros(rows(yt),cols(yt));
      i=1;
      do while i<=cols(yt);
      dy[.,i] = dfft(yy[.,i])*rows(yy);
      i=i+1;
      endo;
      ij=1;
      do while ij<=n;
      perd[((ij-1)*dimy+1):(ij*dimy),.]=vec(dy[ij,.])*conj(dy[ij,.])/(2*pi*rows(yy));
      ij=ij+1;
      endo;
      
      @compute the tests@
      ftesto=fstat0(dpara,yt,perd,fmat,sp_mat0,Dfmat,H,steady);
      b_ftesto=b_fstat0(dpara,yt,perd,fmat,Dfmat,b_H,startf,endf);
     
      
       if (abs(ftesto)>=cdfchii(0.95,np-3) and steady==0) or (abs(ftesto)>=cdfchii(0.95,np-1) and steady==1) ;
        rejfo=rejfo+1;
        endif;

        if (abs(b_ftesto)>=cdfchii(0.95,np-3) and steady==0) or (abs(b_ftesto)>=cdfchii(0.95,np-3) and steady==1) ;
        rejfbo=rejfbo+1;
        endif;

      
         if (abs(ftesto)>=cdfchii(0.90,np-3) and steady==0) or (abs(ftesto)>=cdfchii(0.90,np-1) and steady==1) ;
        rejfo90=rejfo90+1;
        endif;

        if (abs(b_ftesto)>=cdfchii(0.90,np-3) and steady==0) or (abs(b_ftesto)>=cdfchii(0.90,np-3) and steady==1) ;
        rejfbo90=rejfbo90+1;
        endif;

      simu=simu+1;

else;

print "error";

endif;

endo;



 
print "-----sample size----" n;
print "full spectrum: 95%" rejfo/simu;
print "band spectrum: 95%" rejfbo/simu;

print "full spectrum: 90%" rejfo90/simu;
print "band spectrum: 90%" rejfbo90/simu;

jk=jk+1;
endo;



