/* This procedure produces MCMC draws to obtain the confidence intervals.
   It can be used to replicate the second column (Full spectrum)in Table 9.
   For each simulated series, the confidence interval is obtained by running 20 chains (nblock=20) each returning 2000 draws (ntimes=2000) inside of the critical value.
   It takes a day or two to finish depending on the speed of your computer.
   To get confidence intervals for, say 100, simulated series (totalsim=100), it is important to run the program parallelly on multiple processors,
   or substantially decrease the number of draws for each series.
   The default setting is totalsim=1.
*/

new;

#include fstats.src
#include nkmp_mod.src
#include nkmp_spec.g
#include nkmp_sim.src
dlibrary -a PszTgSen; 

outwidth 256;
format /mat /on /mb1 /ros 16,8;

_svdtol=1E-10;

output file=out-2.out on;

ltheta ="pa-full-narrow2";
open fhltheta = ^ltheta for read;

thetasim = readr( fhltheta, 11000 );

ll="lower-bound2";
uu="upper-bound2";
drw="mcmc-draws";

create lowerb = ^ll with THETA, 13, 8;
create upperb = ^uu with THETA, 13, 8;
create mcmc= ^drw with THETA, 13,8;    
/* Load model parameters */

tau        = 2;
kap        = 0.15;

psi1       = 1.5;
psi2       = 1.00;

rhoR       = 0.60;
rhog       = 0.95;
rhoz       = 0.65;

rr_steady  = 0.40;
pi_steady  = 4.00;
gam_steady = 0.50;

sigR       = 0.2;
sigg       = 0.8;
sigz       = 0.45;


spara=zeros(13,1);

spara[1]=tau;
spara[2]=kap;
spara[3]=psi1;
spara[4]=psi2;
spara[5]=rhoR;
spara[6]=rhog;
spara[7]=rhoz;
spara[8]=rr_steady;
spara[9]=pi_steady;
spara[10]=gam_steady;
spara[11]=sigR;
spara[12]=sigg;
spara[13]=sigz;

//load bounds on candidate parameters

btau           =  1E-5~5;  
bkap           =  0~ 1;

bpsi1          =  0 ~ 5; 
bpsi2          =  0~  2;

brhoR          =  0~  0.9;
brhog          =  0~  0.99;
brhoz          =  0~  0.99;

brr_steady     =  0~    5;
bpi_steady     =  0~    20;
bgam_steady    =  0~    5;

bsigR          =   1E-5~  2;
bsigg          =   1E-5~  2;
bsigz          =   1E-5~  2;

cpara=btau|bkap|bpsi1|bpsi2|brhoR|brhog|brhoz|brr_steady|bpi_steady|bgam_steady|bsigR|bsigg|bsigz;

crange=cpara[.,2]-cpara[.,1];

steady=0;

msel=zeros(2,1);
msel[1]=1; 
msel[2]=3; 

dini=1;

rndseed 12345;

totalsim=1; @--the number of simulated series--@

Tsimburn=500;
Tsimtotal=1000;

l90=zeros(totalsim,13);
u90=zeros(totalsim,13);


nrep=1;
do while nrep<=totalsim;

yt = dsgesim(spara,msel,Tsimburn,Tsimtotal); 

yt=yt[Tsimburn+1:Tsimtotal,.];
yt=yt[1:240,.];

n=rows(yt); @sample size@
dimy=cols(yt); @number of variables@


@compute the periodgram@
yy=yt; 
                  
@compute the periodogram, starting at frequency zero@
 perd=zeros(dimy*n,dimy);
 dy=zeros(rows(yt),cols(yt));
 i=1;
 do while i<=cols(yt);
       dy[.,i] = dfft(yy[.,i])*rows(yy);
       i=i+1;
 endo;
 ij=1;
 do while ij<=n;
         perd[((ij-1)*dimy+1):(ij*dimy),.]=vec(dy[ij,.])*conj(dy[ij,.])/(2*pi*rows(yy));
         ij=ij+1;
  endo;



nblock=20; //number of chains
ntimes=2000; //number of draws for each chain
mcd=zeros(nblock*ntimes,13);

startf=round(n/32); @the frequency band@
endf=round(n/6);

block=1;
do while block<=nblock;

@draw an initial value from a prior distribution@
dpara=thetasim[dini,.]';

prop_dpara=dpara;
lk_ini=-1e10;
if steady==1;
np=13;
else;
np=11;
endif;


ftest=100;

totali=1;
acci=1;


drawj=1;

do while drawj<=ntimes;

  @Start with an initial value, perturb it, apply the MCMC, until finding a value within the confidence interval, then increase drawj by one@
  @Here the initial value is taken from some prior distribution; other values can also be used@
   
   if drawj<=1;
        paraj=1;
        do while paraj<=13;
    
            if paraj==9 or paraj==10;@skip the steady state parameters@
                paraj=paraj+1;
            else;
    
            prop_dpara[paraj]=dpara[paraj]+0.05*rndn(1,1)*crange[paraj];  
            if (prop_dpara[paraj]>cpara[paraj,2] or prop_dpara[paraj]<cpara[paraj,1]);
                prop_dpara[paraj]=dpara[paraj];
            endif;
            if acci>=2000;
            goto flip;@switch to a different initial value to avoid getting stuck@
            endif;

            if prop_dpara[paraj]/=dpara[paraj];
                    {H,b_H,Dfmat,fmat,sp_mat0,retcode}=HDmat(prop_dpara,n,msel,startf,endf,steady); 
                    if retcode==0;
                            totali=totali+1;
                            ftest=fstat0(prop_dpara,yt,perd,fmat,sp_mat0,Dfmat,H,steady);
                       
                            lk_prop=-ftest/2;
                            r=abs(exp(lk_prop-lk_ini));
                            
                            dr=rndu(1,1);
                            if dr<=r;
                                print "&&" abs(ftest)~r~prop_dpara'~drawj;
                                acci=acci+1;
                                if  abs(ftest)<=cdfchii(0.90,np-1) and sumc(abs(prop_dpara-dpara))>0;
                                       
                                        mcd[ntimes*(block-1)+drawj,.]=prop_dpara';
                                        drawj=drawj+1;
                                endif;
                                dpara=prop_dpara;
                                lk_ini=lk_prop;
                            else;
                            prop_dpara=dpara;   
                            endif;
                            paraj=paraj+1;
                    else;
                            prop_dpara=dpara;
                    endif;
            endif;
        endif;
    endo;
endif;

@now proceed to obtain more draws@
zzz=rndu(1,1);

@depending on zzz, the procedure switches between two proposal distributions@

@the first distribution generates travels across potential ridges@
if real(abs(lk_ini))<=1e15 and zzz>0.1;
    {H,b_H,Dfmat,fmat,sp_mat0,retcode}=HDmat(dpara,n,msel,startf,endf,steady); 
    H=b_H;
    HH=(H[1:8,1:8]~H[1:8,11:13])|(H[11:13,1:8]~H[11:13,11:13]);
    bigv=invpd(real(HH)+(1e-1)*eye(np));
    coin=rndu(1,1);
    if coin<=0.33;
    ssize=0.05;@further switches between different step sizes@
    elseif coin>0.33 and coin<=0.66;
    ssize=0.1;
    else;
    ssize=0.02;
    endif;

    coin2=rndu(1,1);@travels in different directions@
    if coin2<=0.5;
    pro0=ssize*chol(bigv)'rndn(np,1);
    else;
    pro0=ssize*sqrt(diagrv(eye(11),diag(bigv)))'rndn(np,1);
    endif;
   
    prop_dpara[1:8]=dpara[1:8]-pro0[1:8];
    prop_dpara[11:13]=dpara[11:13]-pro0[9:11];

    if (sumc(prop_dpara.>cpara[.,2])+sumc(prop_dpara.<cpara[.,1]))>=1;
            prop_dpara=dpara;
    elseif sumc(prop_dpara./=dpara)>=1; 
            {H,b_H,Dfmat,fmat,sp_mat0,retcode}=HDmat(prop_dpara,n,msel,startf,endf,steady);
            if retcode==0;
                    totali=totali+1;
                    ftest=fstat0(prop_dpara,yt,perd,fmat,sp_mat0,Dfmat,H,steady);

                    //perform a eigenvalue decomposition of the H matrix
                    lk_prop=-ftest/2;
                    r=abs(exp(lk_prop-lk_ini));
                    dr=rndu(1,1);
                    if dr<=r;
                            acci=acci+1;
                            print "rr" abs(ftest)~r~prop_dpara'~drawj;
                            if abs(ftest)<=cdfchii(0.90,np-1) and sumc(prop_dpara./=dpara)>=1;

                                    mcd[ntimes*(block-1)+drawj,.]=prop_dpara';
                                    drawj=drawj+1;
                            endif;
                            dpara=prop_dpara;
                            lk_ini=lk_prop;
                    else;
                            dpara=dpara;
                            prop_dpara=dpara;
                    endif;
            else;
                    prop_dpara=dpara;
            endif;

    endif;   

endif;   


@the second distribution generates draws travelling along potential ridges@
if real(ftest)<=cdfchii(0.90,np-1) and zzz<=0.1;
    {H,b_H,Dfmat,fmat,sp_mat0,retcode}=HDmat(dpara,n,msel,startf,endf,steady);
    H=b_H;
    HH=(H[1:8,1:8]~H[1:8,11:13])|(H[11:13,1:8]~H[11:13,11:13]);
     bigv=real(HH);
    {va,ve}=eigrs2(bigv); 
    ds=rndu(1,1);
    sgn=-1*(ds>=0.5)+1*(ds<0.5);@switch directions@
    coin=rndu(1,1);
    if coin<=0.33;
    ssize=1;@different step sizes@
    elseif coin>0.33 and coin<=0.66;
    ssize=2;
    else;
    ssize=0.5;
    endif;
    prop_dpara[1:8]=dpara[1:8]+ssize*ve[1:8,1]*sgn;
    prop_dpara[11:13]=dpara[11:13]+ssize*ve[9:11,1]*sgn;
    if (sumc(prop_dpara.>cpara[.,2])+sumc(prop_dpara.<cpara[.,1]))>=1;
            prop_dpara=dpara;
    else;
            {H,b_H,Dfmat,fmat,sp_mat0,retcode}=HDmat(prop_dpara,n,msel,startf,endf,steady);
            if retcode==0;
                        totali=totali+1;
                        ftest=fstat0(prop_dpara,yt,perd,fmat,sp_mat0,Dfmat,H,steady);
                        lk_prop=-ftest/2;
                        r=abs(exp(lk_prop-lk_ini));
                        dr=rndu(1,1);
                        // print real(ftest)~r~prop_dpara';
                        if dr<=r;
                                print "xx" abs(ftest)~r~prop_dpara'~drawj;
                                acci=acci+1;
                                if  abs(ftest)<=cdfchii(0.90,np-1) and sumc(prop_dpara./=dpara)>=1;

                                mcd[ntimes*(block-1)+drawj,.]=prop_dpara';
                                drawj=drawj+1;
                                endif;
                                dpara=prop_dpara;
                                lk_ini=lk_prop;
                        else;
                            prop_dpara=dpara;
                        endif;
            else;
                      prop_dpara=dpara;               
            endif;

    endif;  
endif;

endo;

block=block+1;

flip: 

dini=dini+1;

endo;

wr=writer(mcmc,mcd);

ncol=1;
do while ncol<=13;
mcd[.,ncol]=sortc(mcd[.,ncol],1);
ncol=ncol+1;
endo;
l90[nrep,.]=mcd[1,.];
u90[nrep,.]=mcd[rows(mcd),.];

wr = writer(lowerb,l90[nrep,.]);
wr = writer(upperb,u90[nrep,.]);

print nrep~l90[nrep,.];
print nrep~u90[nrep,.];

nrep=nrep+1;
endo;

closeall lowerb, upperb,fhltheta,mcmc;

