/*This procedure computes the null rejection frequencies under Student t innovations. It replicates the first panel in Table 10.
 Set steady=0 to get "BC frequencies" and "Full spectrum", set steady=1 to get "Mean and spectrum".
 You can experiment with different degrees of freedoms by changing df.
 You should not have to modify the rest. The simulation may take a few hours to finish. */


new;

/*-----------------------------------------*/

@experiment with different degrees of freedom@
//df={5.9,8.3,5.5};
//df={8.2,11.4,7.5};
df={8.1,7.6,5.6};

@set steady to 0 or 1@
steady=0;
/*------------------------------------------*/

lib user gensys.src
#include nkmp_mod.src
#include nkmp_spec.g
#include fstats.src

dlibrary -a PszTgSen; 
outwidth 256;
format /mat /on /mb1 /ros 16,8;

rndseed 12345;
_svdtol=1e-10;


@select the model specification@
msel=zeros(2,1);
msel[1]=1; 
msel[2]=3; 


tau        = 2;
kap        = 0.15;

psi1       = 1.5;
psi2       = 1.00;

rhoR       = 0.60;
rhog       = 0.95;
rhoz       = 0.65;

rr_steady  = 0.40;
pi_steady  = 4.00;
gam_steady = 0.50;

sigR       = 0.2;
sigg       = 0.8;
sigz       = 0.45;

spara=zeros(13,1);
spara[1]=tau;
spara[2]=kap;
spara[3]=psi1;
spara[4]=psi2;
spara[5]=rhoR;
spara[6]=rhog;
spara[7]=rhoz;
spara[8]=rr_steady;
spara[9]=pi_steady;
spara[10]=gam_steady;
spara[11]=sigR;
spara[12]=sigg;
spara[13]=sigz;

np=16;  @the number of parameters@

nn={80,160,240,320};

jk=1;
do while jk<=4;

rejf=0;  @store the rejection frequency of the parameter restriction test@
rejfb=0;
rejfb95=0;
rejf95=0;

simu=1;
Tsimburn=500;
Tsimtotal=1000;

do while simu<=5000;
dpara=spara;

@generating simulated series driving by student t errors@
yt = dsgesim_t(spara,msel,Tsimburn,Tsimtotal,df); 

yt=yt[Tsimburn+1:Tsimtotal,.];
yt=yt[1:nn[jk],.];
n=rows(yt);          @sample size@

startf=round(n/32); @the frequency band@
endf=round(n/6);

dimy=cols(yt);  @number of variables@

{H,b_H,Dfmat,fmat,sp_mat0,retcode}=Hmat_alt(dpara,n,msel,startf,endf,steady,df);
      if retcode==0;
      mu=zeros(3,1);
      mu[1]=dpara[10];
      mu[2]=dpara[9];
      mu[3]=dpara[8]+dpara[9]+4*dpara[10];
      yy=yt; 
      yy[.,1]=yt[.,1]-mu[1]; @demean@
      yy[.,2]=yt[.,2]-mu[2];
      yy[.,3]=yt[.,3]-mu[3];

      @compute the periodogram, starting at frequency zero@
      perd=zeros(dimy*n,dimy);
      dy=zeros(rows(yt),cols(yt));
      i=1;
      do while i<=cols(yt);
      dy[.,i] = dfft(yy[.,i])*rows(yy);
      i=i+1;
      endo;
      ij=1;
      do while ij<=n;
      perd[((ij-1)*dimy+1):(ij*dimy),.]=vec(dy[ij,.])*conj(dy[ij,.])/(2*pi*rows(yy));
      ij=ij+1;
      endo;
      
      ftest=fstatt(dpara,yt,perd,fmat,sp_mat0,Dfmat,H,steady);

      b_ftest=b_fstatt(dpara,yt,perd,fmat,Dfmat,b_H,startf,endf);
      

        if (abs(ftest)>=cdfchii(0.90,np-3) and steady==0) or (abs(ftest)>=cdfchii(0.90,np-1) and steady==1);
        rejf=rejf+1;
        endif;

       if abs(b_ftest)>=cdfchii(0.90,np-3) and steady==0;
        rejfb=rejfb+1;
        endif;

       if (abs(ftest)>=cdfchii(0.95,np-3) and steady==0) or (abs(ftest)>=cdfchii(0.95,np-1) and steady==1);
        rejf95=rejf95+1;
        endif;

       if abs(b_ftest)>=cdfchii(0.95,np-3) and steady==0;
        rejfb95=rejfb95+1;
        endif;


      simu=simu+1;
else;

print "wrong";

endif;

    endo;


print "the degrees of freedom" df;
print "sample size" n;
print "full spectrum: 90% 95%" rejf/simu  rejf95/simu;
print "band spectrum: 90% 95%" rejfb/simu  rejfb95/simu;

jk=jk+1;
endo;


