/*This procedure computes the size adjusted power. It replicates the second panel in Table 10.
 Set pct=0.2 to replicate the first half; set pct=0.4 for the second half.
 In each case, set steady=0 to get "BC frequencies" and "Full spectrum", set steady=1 to get "Mean and spectrum". 
 You should not have to modify the rest. The simulation may take more than a few hours to finish. */


new;

/*-----------------------------------------*/

@experiment with different degrees of freedom@
//df={5.9,8.3,5.5};
//df={8.2,11.4,7.5};
df={8.1,7.6,5.6};

@set steady to 0 or 1@
steady=0;

@set the percentage of deviation@
pct=0.2;
/*------------------------------------------*/



lib user gensys.src
#include nkmp_mod.src
#include nkmp_spec.g
#include fstats.src

dlibrary -a PszTgSen; 
outwidth 256;
format /mat /on /mb1 /ros 16,8;
format /ld 6,4;
_svdtol=1e-10;

//select the model specification

msel=zeros(2,1);
msel[1]=1; 
msel[2]=3; 

/* Load model parameters*/

tau        = 2;
kap        = 0.15;

psi1       = 1.5;
psi2       = 1.00;

rhoR       = 0.60;
rhog       = 0.95;
rhoz       = 0.65;

rr_steady  = 0.40;
pi_steady  = 4.00;
gam_steady = 0.50;

sigR       = 0.2;
sigg       = 0.8;
sigz       = 0.45;


spara=zeros(13,1);
spara[1]=tau;
spara[2]=kap;
spara[3]=psi1;
spara[4]=psi2;
spara[5]=rhoR;
spara[6]=rhog;
spara[7]=rhoz;
spara[8]=rr_steady;
spara[9]=pi_steady;
spara[10]=gam_steady;
spara[11]=sigR;
spara[12]=sigg;
spara[13]=sigz;

longpara=zeros(16,1);
longpara[1:13]=spara;
longpara[14:16]=df;

np=16;  @the number of parameters@


Tsimburn=500;
Tsimtotal=1000;

nn={80,160,240,320};

nsimu=5000;
jk=1;
do while jk<=4;

//first get the cv for the finite sample distribution


f=zeros(nsimu,1);
bf=zeros(nsimu,1);
f90=0;
f95=0;
bf90=0;
bf95=0;


pdraw=1;
do while pdraw<=nsimu;

yt = dsgesim_t(spara,msel,Tsimburn,Tsimtotal,df); 
yt=yt[Tsimburn+1:Tsimtotal,.];
yt=yt[1:nn[jk],.];
n=rows(yt);     @sample size@
startf=round(n/32); @the frequency band@
endf=round(n/6);
dimy=cols(yt);  @number of variables@
{H,b_H,Dfmat,fmat,sp_mat0,retcode}=Hmat_alt(spara,n,msel,startf,endf,steady,df);
    

      mu=zeros(3,1);
      mu[1]=spara[10];
      mu[2]=spara[9];
      mu[3]=spara[8]+spara[9]+4*spara[10];
      yy=yt; 
      yy[.,1]=yt[.,1]-mu[1]; @demean@
      yy[.,2]=yt[.,2]-mu[2];
      yy[.,3]=yt[.,3]-mu[3];

      @compute the periodogram, starting at frequency zero@
      perd=zeros(dimy*n,dimy);
      dy=zeros(rows(yt),cols(yt));
      i=1;
      do while i<=cols(yt);
      dy[.,i] = dfft(yy[.,i])*rows(yy);
      i=i+1;
      endo;
      ij=1;
      do while ij<=n;
      perd[((ij-1)*dimy+1):(ij*dimy),.]=vec(dy[ij,.])*conj(dy[ij,.])/(2*pi*rows(yy));
      ij=ij+1;
      endo;
      
     f[pdraw]=fstatt(spara,yt,perd,fmat,sp_mat0,Dfmat,H,steady);
 
      bf[pdraw]=b_fstatt(spara,yt,perd,fmat,Dfmat,b_H,startf,endf);

pdraw=pdraw+1;
endo;

f=sortc(real(f),1);
bf=sortc(real(bf),1);
f90=f[rows(f)*0.9];
bf90=bf[rows(bf)*0.9];
f95=f[rows(f)*0.95];
bf95=bf[rows(bf)*0.95];


print "sample size" n;
print "critical values:full spectrum" f90 f95 ;
print "critical values: band spectrum" bf90 bf95 ;


rejf=0;  
rejfb=0;
rejf90=0;  @store the rejection frequencies@
rejfb90=0;
simu=1;

pdraw=1;
simu=pdraw;
do while simu<=nsimu;

drawing:
ru=ceil(rndu(1,1)*16);

if steady==0 and ru>=9 and ru<=10;

goto drawing;
endif;


rchg=zeros(16,1);
rchg[ru]=longpara[ru];
pertb=(rndu(1,1)<=0.5)*2-1;

dpara=longpara-pct*pertb*rchg;

{H,b_H,Dfmat,fmat,sp_mat0,retcode}=Hmat_alt(dpara[1:13],n,msel,startf,endf,steady,dpara[14:16]);
   if retcode==0 and dpara[14]>4 and dpara[15]>4 and dpara[16]>4;

@generating simulated series using AS's code@
yt = dsgesim_t(dpara[1:13],msel,Tsimburn,Tsimtotal,dpara[14:16]); 

yt=yt[Tsimburn+1:Tsimtotal,.];
yt=yt[1:nn[jk],.];
n=rows(yt);     @sample size@

startf=round(n/32); @the frequency band@
endf=round(n/6);

dimy=cols(yt);  @number of variables@

{H,b_H,Dfmat,fmat,sp_mat0,retcode}=Hmat_alt(spara[1:13],n,msel,startf,endf,steady,df);

      mu=zeros(3,1);
      mu[1]=spara[10];
      mu[2]=spara[9];
      mu[3]=spara[8]+spara[9]+4*spara[10];
      yy=yt; 
      yy[.,1]=yt[.,1]-mu[1]; @demean@
      yy[.,2]=yt[.,2]-mu[2];
      yy[.,3]=yt[.,3]-mu[3];

      @compute the periodogram, starting at frequency zero@
      perd=zeros(dimy*n,dimy);
      dy=zeros(rows(yt),cols(yt));
      i=1;
      do while i<=cols(yt);
      dy[.,i] = dfft(yy[.,i])*rows(yy);
      i=i+1;
      endo;
      ij=1;
      do while ij<=n;
      perd[((ij-1)*dimy+1):(ij*dimy),.]=vec(dy[ij,.])*conj(dy[ij,.])/(2*pi*rows(yy));
      ij=ij+1;
      endo;
      
     ftest=fstatt(spara[1:13],yt,perd,fmat,sp_mat0,Dfmat,H,steady);
 
     b_ftest=b_fstatt(spara[1:13],yt,perd,fmat,Dfmat,b_H,startf,endf);
     
       
        
        if (abs(ftest)>=f95 and steady==0) or (abs(ftest)>=f95 and steady==1) ;
        rejf=rejf+1;
        endif;

        if (abs(b_ftest)>=bf95 and steady==0) ;
        rejfb=rejfb+1;
        endif;

        if (abs(ftest)>=f90 and steady==0) or (abs(ftest)>=f90 and steady==1) ;
        rejf90=rejf90+1;
        endif;

        if (abs(b_ftest)>=bf90 and steady==0);
        rejfb90=rejfb90+1;
        endif;



      simu=simu+1;
else;
goto drawing;

endif;

    endo;


print "sample size" n;
print "full spectrum: 95%" rejf/simu ;
print "band spectrum: 95%" rejfb/simu ;


print "full spectrum: 90%" rejf90/simu ;
print "band spectrum: 90%" rejfb90/simu ;

jk=jk+1;
endo;