/*This is a procedure that computes the size of the test under 
  low frequency misspecifications. It replicates Table 11.
  Set Case to 1, 2, and 3 for the three types of misspcification.
  The default setting produces results for "BC" and "Full spectrum". Change endf=n/2 to get "BC+High".
 You should not have to modify the rest. The simulation may take a few hours to finish.  */

new;

/*-----------------------*/
         Case=3;
/*-----------------------*/

lib user gensys.src;
#include nkmp_mod.src
#include nkmp_spec.g
#include nkmp_sim.src
#include fstats.src

library pgraph, user;

dlibrary -a PszTgSen; 
outwidth 256;
format /mat /on /mb1 /ros 16,8;
_svdtol=1e-10;

load level263[]=level263.txt;
load level320[]=level320.txt;

rndseed 12345;

multi=1;
@select the model specification@
msel=zeros(2,1);
msel[1]=1; 
msel[2]=3; 

np=13;  @the number of parameters@
steady=0;

tau        = 2;
kap        = 0.15;

psi1       = 1.5;
psi2       = 1.00;

rhoR       = 0.60;
rhog       = 0.95;
rhoz       = 0.65;

rr_steady  = 0.40;
pi_steady  = 4.00;
gam_steady = 0.50;

sigR       = 0.2;
sigg       = 0.8;
sigz       = 0.45;

spara=zeros(13,1);
spara[1]=tau;
spara[2]=kap;
spara[3]=psi1;
spara[4]=psi2;
spara[5]=rhoR;
spara[6]=rhog;
spara[7]=rhoz;
spara[8]=rr_steady;
spara[9]=pi_steady;
spara[10]=gam_steady;
spara[11]=sigR;
spara[12]=sigg;
spara[13]=sigz;

dpara=spara;

nn={80,160,240,320};

jk=1;
do while jk<=4;


rejfo=0;  
rejfbo=0;


rejfo90=0;  
rejfbo90=0;


simu=1;

do while simu<=5000;


@generating simulated series using AS's code@
yt = dsgesim(spara,msel,Tsimburn,Tsimtotal); 

yt=yt[Tsimburn+1:Tsimtotal,.];
yt=yt[1:nn[jk],.];
n=rows(yt);     @sample size@

if case==1 or case==3;
if rows(yt)>104;
br=104;
yt[br+1:rows(yt),1]=yt[br+1:rows(yt),1]-multi*1.078*0.2356/0.9695;
yt[br+1:rows(yt),3]=yt[br+1:rows(yt),3]-multi*4*1.078*0.2356/0.9695;
endif;
endif;

if case==2 or case==3;
  if  n==80;

    yt[.,2]=yt[.,2]+multi*0.275*level263[1:80]/6.43;
    yt[.,3]=yt[.,3]+multi*0.275*level263[1:80]/6.43;
  endif;
if  n==160;
    yt[.,2]=yt[.,2]+multi*0.275*level263[1:160]/6.43;
    yt[.,3]=yt[.,3]+multi*0.275*level263[1:160]/6.43;
  endif;
if  n==240;
    yt[.,2]=yt[.,2]+multi*0.275*level263[1:240]/6.43;
    yt[.,3]=yt[.,3]+multi*0.275*level263[1:240]/6.43;
  endif;
if  n==320;

    yt[.,3]=yt[.,3]+multi*0.275*level320/6.43;
    yt[.,2]=yt[.,2]+multi*0.275*level320/6.43;
  endif;
endif;

    
startf=n/32; @the frequency band@
endf=n/6; 
//endf=n/2; @for BC+High@

dimy=cols(yt);  @number of variables@

{H,b_H,Dfmat,fmat,sp_mat0,retcode}=HDmat(dpara,n,msel,startf,endf,steady);
   
      mu=zeros(3,1);
      mu[1]=dpara[10];
      mu[2]=dpara[9];
      mu[3]=dpara[8]+dpara[9]+4*dpara[10];
      yy=yt; 
      yy[.,1]=yt[.,1]-mu[1]; @demean@
      yy[.,2]=yt[.,2]-mu[2];
      yy[.,3]=yt[.,3]-mu[3];

      @compute the periodogram, starting at frequency zero@
      perd=zeros(dimy*n,dimy);
      dy=zeros(rows(yt),cols(yt));
      i=1;
      do while i<=cols(yt);
      dy[.,i] = dfft(yy[.,i])*rows(yy);
      i=i+1;
      endo;
      ij=1;
      do while ij<=n;
      perd[((ij-1)*dimy+1):(ij*dimy),.]=vec(dy[ij,.])*conj(dy[ij,.])/(2*pi*rows(yy));
      ij=ij+1;
      endo;
      
     
      ftesto=fstat0(dpara,yt,perd,fmat,sp_mat0,Dfmat,H,steady);

      b_ftesto=b_fstat0(dpara,yt,perd,fmat,Dfmat,b_H,startf,endf);
      
        
       if abs(ftesto)>=cdfchii(0.95,10) ;
        rejfo=rejfo+1;
        endif;

        if abs(b_ftesto)>=cdfchii(0.95,10);
        rejfbo=rejfbo+1;
        endif;

      
       if abs(ftesto)>=cdfchii(0.90,10) ;
        rejfo90=rejfo90+1;
        endif;

        if abs(b_ftesto)>=cdfchii(0.90,10);
        rejfbo90=rejfbo90+1;
        endif;


      simu=simu+1;


endo;


print "sample size" n;

print "full spectrum: 95%" rejfo/simu;
print "band spectrum: 95%" rejfbo/simu;

print "full spectrum: 90%" rejfo90/simu ;
print "band spectrum: 90%" rejfbo90/simu ;



jk=jk+1;
endo;


